local importPath=string.gsub(getfenv(1)._.Name,"%.Calendar","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

-- requires fontMetric
if FontMetric==nil then
	import (importPath.."FontSupport")
	fontMetric=FontMetric()
end
-- a calendar style date selection and date functions
-- note, we don't have access to any of the os.date functions :(
isLeapYear=function(year)
	ret=({math.modf(year/4)})[2]==0
	if ret and (({math.modf(year/100)})[2]==0 and ({math.modf(year/400)})[2]~=0) then ret=false end
	return ret
end
getTimeZone=function(checkDST)
	-- returns offset from UTC
	local timeZone=(Turbine.Engine:GetLocalTime()-getEpochTime())/3600
	if checkDST and Turbine.Engine:GetDate().IsDST then timeZone=timeZone+1 end
	return timeZone
end
getEpochTime=function(dateTime,useCurrentTime)
	-- only Month, Day, Year are required. if Hour, Minute, or Second are nil they will be replaced with current values
	if dateTime==nil then dateTime=Turbine.Engine:GetDate() end
	if useCurrentTime==nil then useCurrentTime=true end
	local currentDate=Turbine.Engine:GetDate()
	local seconds=0
	dateTime.Year=tonumber(dateTime.Year)
	dateTime.Month=tonumber(dateTime.Month)
	dateTime.Day=tonumber(dateTime.Day)
	if dateTime.Year==nil or dateTime.Year~=math.floor(dateTime.Year) or dateTime.Year<1970 then
		if Settings.Debug then error("Invalid date.Year:"..tostring(dateTime.Year)) end
	elseif dateTime.Month==nil or dateTime.Month~=math.floor(dateTime.Month) or dateTime.Month<1 or dateTime.Month>12 then
		if Settings.Debug then error("Invalid date.Month:"..tostring(dateTime.Month)) end
	elseif dateTime.Day==nil or dateTime.Day~=math.floor(dateTime.Day) or dateTime.Day<1 then
		if Settings.Debug then error("Invalid date.Day:"..tostring(dateTime.Day)) end
	else
		local monthDays={31,28,31,30,31,30,31,31,30,31,30,31}
		if isLeapYear(dateTime.Year) then
			monthDays={31,29,31,30,31,30,31,31,30,31,30,31}
		end
		if dateTime.Day>monthDays[dateTime.Month] then
			if Settings.Debug then error("Invalid date.Day:"..tostring(dateTime.Day)) end
		else
			if dateTime.Hour==nil then
				if useCurrentTime then
					dateTime.Hour=currentDate.Hour
				else
					dateTime.Hour=0
				end
			end
			if dateTime.Minute==nil then
				if useCurrentTime then
					dateTime.Minute=currentDate.Minute
				else
					dateTime.Minute=0
				end
			end
			if dateTime.Second==nil then
				if useCurrentTime then
					dateTime.Second=currentDate.Second
				else
					dateTime.Second=0
				end
			end
			if dateTime.Hour~=math.floor(dateTime.Hour) or dateTime.Hour<0 or dateTime.Hour>23 then
				if Settings.Debug then error("Invalid date.Hour:"..tostring(dateTime.Hour)) end
			elseif dateTime.Minute~=math.floor(dateTime.Minute) or dateTime.Minute<0 or dateTime.Minute>59 then
				if Settings.Debug then error("Invalid date.Minute:"..tostring(dateTime.Minute)) end
			elseif dateTime.Second~=math.floor(dateTime.Second) or dateTime.Second<0 or dateTime.Second>59 then
				if Settings.Debug then error("Invalid date.Second:"..tostring(dateTime.Second)) end
			else
				seconds=dateTime.Hour*3600+dateTime.Minute*60+dateTime.Second
				-- we have a valid date, now calculate number of seconds
				for tmpYear=1970,dateTime.Year-1 do
					if isLeapYear(tmpYear) then
						seconds=seconds+31622400
					else
						seconds=seconds+31536000
					end
				end
				for tmpMonth=1,dateTime.Month-1 do
					seconds=seconds+monthDays[tmpMonth]*86400
				end
				for tmpDay=1,dateTime.Day-1 do
					seconds=seconds+86400
				end
			end
		end
	end
	return seconds
	-- returns seconds since 1/1/1970 for the specified dateTime
	-- calling it with a nil dateTime is similar to Turbine.Engine:GetLocalTime() except getEpochTime is UTC, so not adjusted for timezone or DST
end
getFirstMonthDay=function(month,year)
	local tmpDow
	if year<1970 or month<1 or month>12 then
		if Settings.Debug then error("Invalid month/year") end
	else
		-- determine number of days from 1/1/1970 (thursday)
		days=0
		local monthDays={31,28,31,30,31,30,31,31,30,31,30,31}
		if isLeapYear(year) then
			monthDays={31,29,31,30,31,30,31,31,30,31,30,31}
		end
		for tmpYear=1970,year-1 do
			if isLeapYear(tmpYear) then
				days=days+366
			else
				days=days+365
			end
		end
		for tmpMonth=1,month-1 do
			days=days+monthDays[tmpMonth]
		end
		_,tmpDow=math.modf((days+4)/7)
	end
	return math.floor(tmpDow*7+.5)+1
end
getEpochTime2=function(date,format)
	-- gets epochtime (UTC) for the start of the given date
	local month,day,year,seconds
	if string.find(date,"-")~=nil then
		if format==nil then
			month,day,year=string.match(date,"(%d+)%-(%d+)%-(%d+)")
		else
			format=string.lower(format)
			if format=="mm-dd-yyyy" then
				month,day,year=string.match(date,"(%d+)%-(%d+)%-(%d+)")
			elseif format=="dd-mm-yyyy" then
				day,month,year=string.match(date,"(%d+)%-(%d+)%-(%d+)")
			elseif format=="yyyy-mm-dd" then
				year,month,day=string.match(date,"(%d+)%-(%d+)%-(%d+)")
			else
				if Settings.Debug then error("Unrecognized date:"..tostring(date).." format:"..tostring(format)) end
				-- unknown format
			end
		end
		
	elseif string.find(date,"/")~=nil then
		if format==nil then
			month,day,year=string.match(date,"(%d+)%/(%d+)%/(%d+)")
		else
			format=string.lower(formtat)
			if format=="mm/dd/yyyy" then
				month,day,year=string.match(date,"(%d+)%/(%d+)%/(%d+)")
			elseif format=="dd/mm/yyyy" then
				day,month,year=string.match(date,"(%d+)%/(%d+)%/(%d+)")
			elseif format=="yyyy/mm/dd" then
				year,month,day=string.match(date,"(%d+)%/(%d+)%/(%d+)")
			else
				if Settings.Debug then error("Unrecognized date:"..tostring(date).." format:"..tostring(format)) end
				-- unknown format
			end
		end
	else
		if Settings.Debug then error("Unrecognized date:"..tostring(date)) end
		-- unknown format?
	end
	year=tonumber(year)
	month=tonumber(month)
	day=tonumber(day)
	if year==nil or month==nil or day==nil or year~=math.floor(year) or month~=math.floor(month) or day~=math.floor(day) then
		if Settings.Debug then error("Invalid date:"..tostring(date)) end
	elseif year<1970 or month<1 or month>12 or day<1 then
		if Settings.Debug then error("Invalid date:"..tostring(date)) end
	else
		local monthDays={31,28,31,30,31,30,31,31,30,31,30,31}
		if isLeapYear(year) then
			monthDays={31,29,31,30,31,30,31,31,30,31,30,31}
		end
		if day>monthDays[month] then
			if Settings.Debug then error("Invalid date:"..tostring(date)) end
		else
			seconds=0
			-- we have a valid date, now calculate number of seconds
			for tmpYear=1970,year-1 do
				if isLeapYear(tmpYear) then
					seconds=seconds+31622400
				else
					seconds=seconds+31536000
				end
			end
			for tmpMonth=1,month-1 do
				seconds=seconds+monthDays[tmpMonth]*86400
			end
			for tmpDay=1,day-1 do
				seconds=seconds+86400
			end
		end
	end
	return seconds
end
--***************************************************************************************
-- Calendar Control
-- This control can be used as a stand-alone control or tied to a DateBox control
-- When used as a stand-alone, you must set the Day, Month, Year and set the Callback function
-- to a function that will read the resulting Day, Month and Year from the control
-- Note: Day, Month and Year are also passed in the args table of the Callback function)
-- Call Calendar:SetLanguage(language) where language 1=en, 2=fr, 3=de (it will default to the current client language)
-- You can set the colors using SetForeColor, SetBackColor and SetTrimColor
--***************************************************************************************
Calendar=class( Turbine.UI.Window )
Calendar.Strings={}
Calendar.Strings[1]={}
Calendar.Strings[1][1]="Month"
Calendar.Strings[1][2]="Day"
Calendar.Strings[1][3]="Year"
Calendar.Strings[1][4]="Sun"
Calendar.Strings[1][5]="Mon"
Calendar.Strings[1][6]="Tue"
Calendar.Strings[1][7]="Wed"
Calendar.Strings[1][8]="Thu"
Calendar.Strings[1][9]="Fri"
Calendar.Strings[1][10]="Sat"
Calendar.Strings[1][11]="January"
Calendar.Strings[1][12]="February"
Calendar.Strings[1][13]="March"
Calendar.Strings[1][14]="April"
Calendar.Strings[1][15]="May"
Calendar.Strings[1][16]="June"
Calendar.Strings[1][17]="July"
Calendar.Strings[1][18]="August"
Calendar.Strings[1][19]="September"
Calendar.Strings[1][20]="October"
Calendar.Strings[1][21]="November"
Calendar.Strings[1][22]="December"
Calendar.Strings[2]={}
Calendar.Strings[2][1]="Mois"
Calendar.Strings[2][2]="Jour"
Calendar.Strings[2][3]="Ann\195\169e"
Calendar.Strings[2][4]="Dim"
Calendar.Strings[2][5]="Lun"
Calendar.Strings[2][6]="Mar"
Calendar.Strings[2][7]="Mer"
Calendar.Strings[2][8]="Jeu"
Calendar.Strings[2][9]="Ven"
Calendar.Strings[2][10]="Sam"
Calendar.Strings[2][11]="Janvier"
Calendar.Strings[2][12]="F\195\169vrier"
Calendar.Strings[2][13]="Mars"
Calendar.Strings[2][14]="Avril"
Calendar.Strings[2][15]="Mai"
Calendar.Strings[2][16]="Juin"
Calendar.Strings[2][17]="Juillet"
Calendar.Strings[2][18]="Ao\195\187t"
Calendar.Strings[2][19]="Septembre"
Calendar.Strings[2][20]="Octobre"
Calendar.Strings[2][21]="Novembre"
Calendar.Strings[2][22]="D\195\169cembre"
Calendar.Strings[3]={}
Calendar.Strings[3][1]="Monat"
Calendar.Strings[3][2]="Tag"
Calendar.Strings[3][3]="Jahr"
Calendar.Strings[3][4]="So"
Calendar.Strings[3][5]="Mo"
Calendar.Strings[3][6]="Di"
Calendar.Strings[3][7]="Mi"
Calendar.Strings[3][8]="Do"
Calendar.Strings[3][9]="Fr"
Calendar.Strings[3][10]="Sa"
Calendar.Strings[3][11]="January"
Calendar.Strings[3][12]="February"
Calendar.Strings[3][13]="March"
Calendar.Strings[3][14]="April"
Calendar.Strings[3][15]="May"
Calendar.Strings[3][16]="June"
Calendar.Strings[3][17]="July"
Calendar.Strings[3][18]="August"
Calendar.Strings[3][19]="September"
Calendar.Strings[3][20]="October"
Calendar.Strings[3][21]="November"
Calendar.Strings[3][22]="December"
function Calendar:Constructor()
	Turbine.UI.Window.Constructor( self )
	self.Back=Turbine.UI.Control()
	self.Back:SetParent(self)
	self.Back:SetSize(self:GetWidth()-2,self:GetHeight()-2)
	self.Back:SetPosition(1,1)
--	self.MonthCaption=Turbine.UI.Label()
--	self.MonthCaption:SetParent(self.Back)
--	self.DayCaption=Turbine.UI.Label()
--	self.DayCaption:SetParent(self.Back)
--	self.YearCaption=Turbine.UI.Label()
--	self.YearCaption:SetParent(self.Back)
	self.YearText=Turbine.UI.Label()
	self.YearText:SetParent(self.Back)
	self.YearBack=Turbine.UI.Control()
	self.YearBack:SetParent(self.Back)
	self.YearBack:SetSize(25,25)
	self.YearBack:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.YearBack:SetBackground(0x41000276)
	self.YearBack.MouseDown=function()
		self.YearBack:SetBackground(0x41000278)
	end
	self.YearBack.MouseUp=function()
		self.YearBack:SetBackground(0x41000276)
	end
	self.YearBack.MouseClick=function()
		self.Year=self.Year-1
		if self.Year<=1970 then
			self.Year=1970
			self.YearBack:SetBackground(0x41000277)
		else
			self.YearBack:SetBackground(0x41000276)
		end
		self.YearText:SetText(self.Year)
		self:Layout()
	end
--0x41000276 normal
--0x41000277 disabled (self.Year=1970)
--0x41000278 pressed

	self.YearNext=Turbine.UI.Control()
	self.YearNext:SetParent(self.Back)
	self.YearNext:SetSize(25,25)
	self.YearNext:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.YearNext:SetBackground(0x41000272)
	self.YearNext.MouseDown=function()
		self.YearNext:SetBackground(0x41000274)
	end
	self.YearNext.MouseUp=function()
		self.YearNext:SetBackground(0x41000272)
	end
	self.YearNext.MouseClick=function()
		self.Year=self.Year+1
		self.YearBack:SetBackground(0x41000278)
		self.YearNext:SetBackground(0x41000272)
		self.YearText:SetText(self.Year)
		self:Layout()
	end
--0x41000272 normal
--0x41000274 pressed
	self.SundayCaption=Turbine.UI.Label()
	self.SundayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.MondayCaption=Turbine.UI.Label()
	self.MondayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.TuesdayCaption=Turbine.UI.Label()
	self.TuesdayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.WednesdayCaption=Turbine.UI.Label()
	self.WednesdayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.ThursdayCaption=Turbine.UI.Label()
	self.ThursdayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.FridayCaption=Turbine.UI.Label()
	self.FridayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.SaturdayCaption=Turbine.UI.Label()
	self.SaturdayCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.Language=1 -- en default
	local date=Turbine.Engine.GetDate()
	self.Month=date.Month
	self.Year=date.Year
	self.Day=date.Day
	self.MonthSelect=DropDownList()
	self.MonthSelect:SetParent(self.Back)
	self.MonthSelect:SetZOrder(1)
	self.Grid=Turbine.UI.ListBox()
	self.Grid:SetParent(self.Back)

	if Turbine.Shell.IsCommand("hilfe") then
		self.Language=3;
	elseif Turbine.Shell.IsCommand("aide") then
		self.Language=2;
	end
	self.Layout=function(self)
		local size=Turbine.UI.Lotro.FontInfo[self.Font].size
		local dayWidth=self.MondayCaption:GetWidth()
		local monthDays={31,28,31,30,31,30,31,31,30,31,30,31}
		if isLeapYear(self.Year) then
			monthDays={31,29,31,30,31,30,31,31,30,31,30,31}
		end
		-- will layout controls using current font and control sizes
		self:SetWidth((dayWidth+3)*7+3)
		self.Back:SetWidth(self:GetWidth()-2)
--		self.MonthCaption:SetPosition(1,1)
		self.MonthSelect:SetPosition(1,1)
		self.YearNext:SetPosition(self.Back:GetWidth()-self.YearNext:GetWidth()-1,self.MonthSelect:GetTop())
		self.YearText:SetPosition(self.YearNext:GetLeft()-self.YearText:GetWidth()-2,self.MonthSelect:GetTop())
		self.YearBack:SetPosition(self.YearText:GetLeft()-self.YearBack:GetWidth()-2,self.MonthSelect:GetTop())
--		self.YearCaption:SetPosition(self.YearText:GetLeft()-self.YearCaption:GetWidth()-5,self.MonthSelect:GetTop())
		self.Grid:SetPosition(1,self.MonthSelect:GetTop()+self.MonthSelect:GetHeight()+5)
		self.Grid:SetWidth(self.Back:GetWidth())
		self.Grid:ClearItems()
		self.Grid:SetHeight(7*(size+1)) -- larger than possible
		local tmpRow=Turbine.UI.Control()
		tmpRow:SetParent(self.Grid)
		tmpRow:SetHeight(size+1)
		tmpRow:SetBackColor(self.TrimColor)
		tmpRow:SetWidth(self.Grid:GetWidth())
		self.SundayCaption:SetParent(tmpRow)
		self.SundayCaption:SetLeft(1)
		self.MondayCaption:SetParent(tmpRow)
		self.MondayCaption:SetLeft(self.SundayCaption:GetLeft()+self.SundayCaption:GetWidth()+3)
		self.TuesdayCaption:SetParent(tmpRow)
		self.TuesdayCaption:SetLeft(self.MondayCaption:GetLeft()+self.MondayCaption:GetWidth()+3)
		self.WednesdayCaption:SetParent(tmpRow)
		self.WednesdayCaption:SetLeft(self.TuesdayCaption:GetLeft()+self.TuesdayCaption:GetWidth()+3)
		self.ThursdayCaption:SetParent(tmpRow)
		self.ThursdayCaption:SetLeft(self.WednesdayCaption:GetLeft()+self.WednesdayCaption:GetWidth()+3)
		self.FridayCaption:SetParent(tmpRow)
		self.FridayCaption:SetLeft(self.ThursdayCaption:GetLeft()+self.ThursdayCaption:GetWidth()+3)
		self.SaturdayCaption:SetParent(tmpRow)
		self.SaturdayCaption:SetLeft(self.FridayCaption:GetLeft()+self.FridayCaption:GetWidth()+3)
		self.Grid:AddItem(tmpRow)
		tmpRow=Turbine.UI.Control()
		tmpRow:SetParent(self.Grid)
		tmpRow:SetBackColor(self.TrimColor)
		tmpRow:SetHeight(size+1)
		tmpRow:SetWidth(self.Grid:GetWidth())
		day=1
		local tmpLabel={}
		for tmpDay=getFirstMonthDay(self.Month,self.Year),7 do
			tmpLabel[tmpDay]=Turbine.UI.Label()
			tmpLabel[tmpDay]:SetParent(tmpRow)
			tmpLabel[tmpDay]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
			tmpLabel[tmpDay]:SetSize(dayWidth,size)
			if day==self.Day then
				tmpLabel[tmpDay]:SetBackColor(self.ForeColor)
				tmpLabel[tmpDay]:SetForeColor(self.BackColor)
			else
				tmpLabel[tmpDay]:SetBackColor(self.BackColor)
				tmpLabel[tmpDay]:SetForeColor(self.ForeColor)
			end
			tmpLabel[tmpDay]:SetFont(self.Font)
			tmpLabel[tmpDay]:SetText(day)
			tmpLabel[tmpDay]:SetLeft((tmpDay-1)*(dayWidth+3))
			tmpLabel[tmpDay].Day=day
			tmpLabel[tmpDay].MouseClick=function(sender,args)
				--select this as the day
				self.Day=sender.Day
				self:SetWantsUpdates(true) -- defer the layout so that we don't squash the doubleclick handler
			end
			tmpLabel[tmpDay].MouseDoubleClick=function(sender,args)
				--select this as the day
				self.Day=sender.Day
				if self.Callback~=nil then
					local args={}
					args.Day=self.Day
					args.Month=self.Month
					args.Year=self.Year
					self.Callback(args)
				end
				self:SetVisible(false)
			end
			day=day+1
		end
		self.Grid:AddItem(tmpRow)
		local rows=2
		while day<=monthDays[self.Month] do
			local tmpLabel={}
			rows=rows+1
			local tmpRow=Turbine.UI.Control()
			tmpRow:SetParent(self.Grid)
			tmpRow:SetBackColor(self.TrimColor)
			tmpRow:SetHeight(size+1)
			tmpRow:SetWidth(self.Grid:GetWidth())
			for tmpDay=1,7 do
				if day<=monthDays[self.Month] then
					tmpLabel[tmpDay]=Turbine.UI.Label()
					tmpLabel[tmpDay]:SetParent(tmpRow)
					tmpLabel[tmpDay]:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
					tmpLabel[tmpDay]:SetSize(dayWidth,size)
					if day==self.Day then
						tmpLabel[tmpDay]:SetBackColor(self.ForeColor)
						tmpLabel[tmpDay]:SetForeColor(self.BackColor)
					else
						tmpLabel[tmpDay]:SetBackColor(self.BackColor)
						tmpLabel[tmpDay]:SetForeColor(self.ForeColor)
					end
					tmpLabel[tmpDay]:SetFont(self.Font)
					tmpLabel[tmpDay]:SetText(day)
					tmpLabel[tmpDay]:SetLeft((tmpDay-1)*(dayWidth+3))
					tmpLabel[tmpDay].Day=day
					tmpLabel[tmpDay].MouseClick=function(sender,args)
						--select this as the day
						self.Day=sender.Day
						self:SetWantsUpdates(true) -- defer the layout so that we don't squash the doubleclick handler
					end
					tmpLabel[tmpDay].MouseDoubleClick=function(sender,args)
						--select this as the day
						self.Day=sender.Day
						if self.Callback~=nil then
							local args={}
							args.Day=self.Day
							args.Month=self.Month
							args.Year=self.Year
							self.Callback(args)
						end
						self:SetVisible(false)
					end
				end
				day=day+1
			end
			self.Grid:AddItem(tmpRow)
		end
		self.Grid:SetHeight(rows*(size+1))
		self.Back:SetHeight(self.Grid:GetTop()+self.Grid:GetHeight())
		self:SetHeight(self.Back:GetTop()+self.Back:GetHeight()+1)
	end
	self.SetFont=function(self,font)
		self.Font=font
		local size=Turbine.UI.Lotro.FontInfo[self.Font].size
		fontMetric:SetFont(font)
--		self.MonthCaption:SetFont(font)
--		self.MonthCaption:SetSize(fontMetric:GetTextWidth(Calendar.Strings[self.Language][1]..": "),size)
--		self.MonthCaption:SetText(Calendar.Strings[self.Language][1]..":")
		-- reload the month list
		self.MonthSelect.SelectedIndexChanged=function()end
		self.MonthSelect:ClearList()
		for k=1,12 do
			self.MonthSelect:AddItem(Calendar.Strings[self.Language][k+10],k)
		end
		self.MonthSelect:SetFont(self.Font)
		local monthWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][11])
		for k=2,12 do
			if fontMetric:GetTextWidth(Calendar.Strings[self.Language][k+10])>monthWidth then monthWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][k+10]) end
		end
		self.MonthSelect:SetWidth(monthWidth+15)
		self.MonthSelect:SetSelectedIndex(self.Month)
		self.MonthSelect.SelectedIndexChanged=self.MonthChanged
--		self.DayCaption:SetFont(font)
--		self.DayCaption:SetSize(fontMetric:GetTextWidth(Calendar.Strings[self.Language][2]..": "),size)
--		self.YearCaption:SetFont(font)
--		self.YearCaption:SetSize(fontMetric:GetTextWidth(Calendar.Strings[self.Language][3]..": "),size)
		self.YearText:SetFont(font)
		self.YearText:SetSize(fontMetric:GetTextWidth("8888 "),size)
		self.YearText:SetText(self.Year)
		local dayWidth=fontMetric:GetTextWidth("88x") -- min width for two digits plus 'x' spacer
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][4])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][4]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][5])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][5]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][6])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][6]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][7])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][7]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][8])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][8]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][9])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][9]) end
		if fontMetric:GetTextWidth(Calendar.Strings[self.Language][10])>dayWidth then dayWidth=fontMetric:GetTextWidth(Calendar.Strings[self.Language][10]) end
		dayWidth=dayWidth+5
		self.SundayCaption:SetFont(font)
		self.SundayCaption:SetSize(dayWidth,size)
		self.SundayCaption:SetText(Calendar.Strings[self.Language][4])
		self.MondayCaption:SetFont(font)
		self.MondayCaption:SetSize(dayWidth,size)
		self.MondayCaption:SetText(Calendar.Strings[self.Language][5])
		self.TuesdayCaption:SetFont(font)
		self.TuesdayCaption:SetSize(dayWidth,size)
		self.TuesdayCaption:SetText(Calendar.Strings[self.Language][6])
		self.WednesdayCaption:SetFont(font)
		self.WednesdayCaption:SetSize(dayWidth,size)
		self.WednesdayCaption:SetText(Calendar.Strings[self.Language][7])
		self.ThursdayCaption:SetFont(font)
		self.ThursdayCaption:SetSize(dayWidth,size)
		self.ThursdayCaption:SetText(Calendar.Strings[self.Language][8])
		self.FridayCaption:SetFont(font)
		self.FridayCaption:SetSize(dayWidth,size)
		self.FridayCaption:SetText(Calendar.Strings[self.Language][9])
		self.SaturdayCaption:SetFont(font)
		self.SaturdayCaption:SetSize(dayWidth,size)
		self.SaturdayCaption:SetText(Calendar.Strings[self.Language][10])
		self:Layout()
	end
	self.SetForeColor=function(self,color)
		self.ForeColor=Turbine.UI.Color(color.A,color.R,color.G,color.B)
--		self.MonthCaption:SetForeColor(color)
--		self.DayCaption:SetForeColor(color)
--		self.YearCaption:SetForeColor(color)
		self.MonthSelect:SetTextColor(color)
		self.YearText:SetForeColor(color)
		self.SundayCaption:SetForeColor(color)
		self.MondayCaption:SetForeColor(color)
		self.TuesdayCaption:SetForeColor(color)
		self.WednesdayCaption:SetForeColor(color)
		self.ThursdayCaption:SetForeColor(color)
		self.FridayCaption:SetForeColor(color)
		self.SaturdayCaption:SetForeColor(color)
	end
	self.SetTrimColor=function(self,color)
		self.TrimColor=Turbine.UI.Color(color.A,color.R,color.G,color.B)
		Turbine.UI.Window.SetBackColor(self,color)
		self.MonthSelect:SetBorderColor(color)
	end
	self.SetBackColor=function(self,color)
		self.BackColor=Turbine.UI.Color(color.A,color.R,color.G,color.B)
		self.Back:SetBackColor(color)
--		self.MonthCaption:SetBackColor(color)
--		self.DayCaption:SetBackColor(color)
--		self.YearCaption:SetBackColor(color)
		self.MonthSelect:SetBackColor(color)
		self.MonthSelect:SetCurrentBackColor(color)
		self.YearText:SetBackColor(color)
		self.SundayCaption:SetBackColor(color)
		self.MondayCaption:SetBackColor(color)
		self.TuesdayCaption:SetBackColor(color)
		self.WednesdayCaption:SetBackColor(color)
		self.ThursdayCaption:SetBackColor(color)
		self.FridayCaption:SetBackColor(color)
		self.SaturdayCaption:SetBackColor(color)
	end
	self.SetLanguage=function(self,language)
		if language==1 or language==2 or language==3 then
			self.Language=language
			self:SetFont(self.Font) -- will set the caption sizes and text
		end
	end
	self:SetForeColor(Turbine.UI.Color.Black)
	self:SetTrimColor(Turbine.UI.Color.Gray)
	self:SetBackColor(Turbine.UI.Color.White)
	self:SetFont(Turbine.UI.Lotro.Font.Verdana20) -- default to verdana 20
	self.MonthChanged=function()
		self.Month=self.MonthSelect:GetValue()
		self:Layout()
	end
	self.MonthSelect.SelectedIndexChanged=self.MonthChanged
	self.Update=function()
		self:SetWantsUpdates(false)
		self:Layout()
		if self.DateChanged~=nil then self.DateChanged() end
	end
	self.SetMonth=function(self,month)
		self.Month=month
		self:Layout()
	end
	self.SetDay=function(self,day)
		self.Day=day
		self:Layout()
	end
	self.SetYear=function(self,year)
		self.Year=year
		self:Layout()
	end
end

--***************************************************************************************
-- DateBox Control
-- This is a self sizing/formatting date control which can call the Calendar control
-- to change the date. The user can also manually chance the Day, Month or Year. When
-- the user changes the date manually or picks a date with the Calendar control, the DateChanged
-- event will be fired (if set)
-- The control will automatically resize itself for the font assigned by DateBox:SetFont(font)
--***************************************************************************************
DateBox = class(Turbine.UI.Control)
if displayWidth==nil then
	displayWidth=Turbine.UI.Display:GetWidth()
end
if displayHeight==nil then
	displayHeight=Turbine.UI.Display:GetHeight()
end
-- create a masked input control with three inputs and functions to setdate and getdate
function isLeapYear(year)
	local ret=false
	year=tonumber(year)
	if year~=nil then
		if math.floor(year/4)==year/4 then
			ret=true
			if math.floor(year/100)==year/100 then
				ret=false
				if math.floor(year/400)==year/400 then
					ret=true
				end
			end
		end
	end
	return ret
end

DateBox.Constructor=function(self)
	Turbine.UI.Control.Constructor( self )
	self.Font=Turbine.UI.Lotro.Font.Verdana18
	self.TextHeight=Turbine.UI.Lotro.FontInfo[self.Font].size
	self:SetHeight(self.TextHeight+2)
	self.TextColor=Turbine.UI.Color(1,1,1)
	self.DateSep="/"
	self.Background=Turbine.UI.Control()
	self.Background:SetParent(self)
	self.Background:SetSize(96,self.TextHeight)
	self.Background:SetPosition(1,1)
	self.MonthText=Turbine.UI.TextBox()
	self.MonthText:SetParent(self.Background)
	self.MonthText:SetPosition(0,0)
	self.MonthText:SetSize(20,self.TextHeight) -- just an initial guess, we can format it better once we implement font sizing
	self.MonthText:SetMarkupEnabled(false) -- no need for seperate colors and only numerics are allowed
	self.MonthText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.MonthText:SetFont(self.Font)
	self.MonthText:SetForeColor(self.TextColor)
	self.MonthText.settingText=false
	self.MonthText.oldText=""
	self.MonthText.KeyDown=function(sender,args)
		if self.MonthText.oldText=="" and args.Action==99 then
			-- do nothing on backspace
		end
	end
	self.MonthText.KeyUp=function(sender,args)
	end
	self.MonthText.FocusGained=function()
		self.MonthText.gainingFocus=true
		self.MonthText:SetWantsKeyEvents(true)
	end
	self.MonthText.FocusLost=function()
		self.MonthText:SetWantsKeyEvents(false)
	end
	self.MonthText.TextChanged=function(sender,args)
		if self.MonthText.gainingFocus then
			self.MonthText.gainingFocus=false
		else
			if not self.MonthText.settingText then
				self.MonthText.settingText=true
				local tmpText=self.MonthText:GetText()
				local newText=""
				-- strip all but numbers
				for i,v in ipairs({tmpText:byte(1,-1)}) do
					if (v >= 48) and (v <=57) then
						newText = newText .. string.char(v)
					end
				end
				if string.len(newText)>2 then
					newText=string.sub(newText,2)
				end
				local num=tonumber(newText)
				if num==nil then
					newText=""
				else
					if num<0 then
						newText="0"
					elseif num>12 then
						newText="12"
					end
				end
				if newText=="00" then newText="01" end
				if newText~=tmpText then
					self.MonthText:SetText(newText)
				end
				self.MonthText.settingText=false
				self.MonthText.oldText=newText
			end
			if string.len(self.MonthText:GetText())==2 then
				self.DayText:Focus()
			end
			self.Month=tonumber(self.MonthText:GetText())
			if self.DateChanged~=nil then self.DateChanged() end
		end
	end

	self.Sep1=Turbine.UI.Label()
	self.Sep1:SetParent(self.Background)
	self.Sep1:SetPosition(self.MonthText:GetLeft()+self.MonthText:GetWidth(),0)
	self.Sep1:SetSize(8,self.TextHeight)
	self.Sep1:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.Sep1:SetForeColor(self.TextColor)
	self.Sep1:SetText(self.DateSep)

	self.DayText=Turbine.UI.TextBox()
	self.DayText:SetParent(self.Background)
	self.DayText:SetPosition(self.Sep1:GetLeft()+self.Sep1:GetWidth(),0)
	self.DayText:SetSize(20,self.TextHeight) -- just an initial guess, we can format it better once we implement font sizing
	self.DayText:SetMarkupEnabled(false) -- no need for seperate colors and only numerics are allowed
	self.DayText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.DayText:SetFont(self.Font)
	self.DayText:SetForeColor(self.TextColor)
	self.DayText.settingText=false
	self.DayText.oldText=""
	self.DayText.KeyDown=function(sender,args)
		if self.DayText.oldText=="" and args.Action==99 then
			self.MonthText.Focus(self.MonthText)
		end
	end
	self.DayText.KeyUp=function(sender,args)
	end
	self.DayText.FocusGained=function()
		self.DayText.gainingFocus=true
		self.DayText:SetWantsKeyEvents(true)
	end
	self.DayText.FocusLost=function()
		self.DayText:SetWantsKeyEvents(false)
	end
	self.DayText.TextChanged=function(sender,args)
		if self.DayText.gainingFocus then
			self.DayText.gainingFocus=false
		else
			if not self.DayText.settingText then
				self.DayText.settingText=true
				local tmpText=self.DayText:GetText()
				local newText=""
				-- strip all but numbers
				for i,v in ipairs({tmpText:byte(1,-1)}) do
					if (v >= 48) and (v <=57) then
						newText = newText .. string.char(v)
					end
				end
				if string.len(newText)>2 then
					newText=string.sub(newText,2)
				end
				local num=tonumber(newText)
				local maxDay=31
				local month=tonumber(self.MonthText:GetText())
				local year=tonumber(self.YearText:GetText())
	
				if month==2 then
					if year==nil then
						maxDay=29
					else
						maxDay=28
					end
					if isLeapYear(year) then
						maxDay=29
					end
				elseif month==4 or month==6 or month==9 or month==11 then
					maxDay=30
				end
	
				if num==nil then
					newText=""
				else
					if num<0 then
						newText="0"
					elseif num>maxDay then
						newText=tostring(maxDay)
					end
				end
				if newText=="00" then newText="01" end
				if newText~=tmpText then
					self.DayText:SetText(newText)
				end
				self.DayText.settingText=false
				self.DayText.oldText=newText
			end
			if string.len(self.DayText:GetText())==2 then
				self.YearText:Focus()
			end
			self.Day=tonumber(self.DayText:GetText())

			if self.DateChanged~=nil then self.DateChanged() end
		end
	end

	self.Sep2=Turbine.UI.Label()
	self.Sep2:SetParent(self.Background)
	self.Sep2:SetPosition(self.DayText:GetLeft()+self.DayText:GetWidth(),0)
	self.Sep2:SetSize(8,self.TextHeight)
	self.Sep2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.Sep2:SetForeColor(self.TextColor)
	self.Sep2:SetText(self.DateSep)

	self.YearText=Turbine.UI.TextBox()
	self.YearText:SetParent(self.Background)
	self.YearText:SetPosition(self.Sep2:GetLeft()+self.Sep2:GetWidth(),0)
	self.YearText:SetSize(40,self.TextHeight) -- just an initial guess, we can format it better once we implement font sizing
	self.YearText:SetMarkupEnabled(false) -- no need for seperate colors and only numerics are allowed
	self.YearText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.YearText:SetFont(self.Font)
	self.YearText:SetForeColor(self.TextColor)
	self.YearText.settingText=false
	self.YearText.oldText=""
	self.YearText.KeyDown=function(sender,args)
		if self.YearText.oldText=="" and args.Action==99 then
			self.DayText.Focus(self.DayText)
		end
	end
	self.YearText.KeyUp=function(sender,args)
	end
	self.YearText.FocusGained=function()
		self.YearText.gainingFocus=true
		self.YearText:SetWantsKeyEvents(true)
	end
	self.YearText.FocusLost=function()
		self.YearText:SetWantsKeyEvents(false)
	end
	self.YearText.TextChanged=function(sender,args)
		if self.YearText.gainingFocus then
			--just suppressing the extraneous TextChanged event when focus is gained
			self.YearText.gainingFocus=false
		else
			if not self.YearText.settingText then
				self.YearText.settingText=true
				local tmpText=self.YearText:GetText()
				local newText=""
				-- strip all but numbers
				for i,v in ipairs({tmpText:byte(1,-1)}) do
					if (v >= 48) and (v <=57) then
						newText = newText .. string.char(v)
					end
				end
				if string.len(newText)>4 then
					newText=string.sub(newText,1,4)
				end
				local num=tonumber(newText)
	
				if num==nil then
					newText=""
				else
					if num<0 then
						newText="0"
					end
				end
				if newText~=tmpText then
					self.YearText:SetText(newText)
				end
				if string.len(self.YearText:GetText())==4 then
					local month=tonumber(self.MonthText:GetText())
					if month==2 then
						-- re-verify leapyear
						if not isLeapYear(self.YearText:GetText()) then
							local day=tonumber(self.DayText:GetText())
							if day>28 then
								self.DayText:SetText("28")
							end
						end
					end
				end
				self.YearText.settingText=false
				self.YearText.oldText=newText
			end
			self.Year=tonumber(self.YearText:GetText())
			if self.DateChanged~=nil then self.DateChanged() end
		end
	end
	self.CalendarEnabled=true
-- create button for a popup calendar control?
	self.CalendarIcon=Turbine.UI.Control()
	self.CalendarIcon:SetParent(self)
	self.CalendarIcon:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.CalendarIcon:SetBackground(resourcePath.."/calendar18x16.jpg")
	self.CalendarIcon:SetSize(18,16)
	self.CalendarIcon:SetPosition(self.YearText:GetLeft()+self.YearText:GetWidth()+2,(self:GetHeight()-self.CalendarIcon:GetHeight())/2)
	self:SetWidth(self.CalendarIcon:GetLeft()+self.CalendarIcon:GetWidth()+2)
	Turbine.UI.Control.SetBackColor(self,Turbine.UI.Color(.5,.5,.1)) -- trim color
	self.Background:SetBackColor(Turbine.UI.Color.Black)
	self.Calendar=Calendar()
	self.CalendarIcon.MouseClick=function(sender,args)
		-- show popup calendar
		local left,top=sender:PointToScreen(args.X,args.Y)
		if left+self.Calendar:GetWidth()>displayWidth then left=displayWidth-self.Calendar:GetWidth() end
		if top+self.Calendar:GetHeight()>displayHeight then top=displayHeight-self.Calendar:GetHeight() end
		self.Calendar:SetPosition(left,top)
		self.Calendar:SetMonth(self.Month)
		self.Calendar:SetDay(self.Day)
		self.Calendar:SetYear(self.Year)
		self.Calendar:SetVisible(true)
		self.Calendar.Callback=self.CalendarChanged
	end
	self.CalendarChanged=function(args)
		self.Month=self.Calendar.Month
		self.Day=self.Calendar.Day
		self.Year=self.Calendar.Year
		self.MonthText:SetText(self.Month)
		self.DayText:SetText(self.Day)
		self.YearText:SetText(self.Year)
		if self.DateChanged~=nil then self.DateChanged() end
	end
	self.SetParent=function(sender,parent)
		-- need to get topmost parent (the containing window)
		local window=parent
		while window:GetParent()~=nil do
			window=window:GetParent()
		end
		self.Calendar:SetZOrder(window:GetZOrder()+1)
		Turbine.UI.Control.SetParent(sender,parent)
	end
end
DateBox.SetCalendarEnabled=function(self,enable)
	-- if enabled, display a calendar icon and when clicking on the DateBox,
	self.CalendarEnabled=enable
	self.CalendarIcon:SetVisible(enable)
	if enable then
		self:SetWidth(self.CalendarIcon:GetLeft())
	else
		self:SetWidth(self.CalendarIcon:GetLeft()+self.CalendarIcon:GetWidth()+2)
	end
end
DateBox.SetFont=function(self,font)
	-- this will change font and automatically resize/reposition the parts of the control
	if Turbine.UI.Lotro.FontInfo[self.Font]~= nil then
		fontMetric:SetFont(font)
		local dateWidth1=fontMetric:GetTextWidth("88")+2
		local dateWidth2=fontMetric:GetTextWidth("8888")+2
		local sepWidth=fontMetric:GetTextWidth(self.DateSep)+2
		self.TextHeight=Turbine.UI.Lotro.FontInfo[font].size
		self.Font=font
		if self.TextHeight>14 then
			self:SetHeight(self.TextHeight+2)
		else
			self:SetHeight(16) -- min height=16
		end
		self.MonthText:SetFont(self.Font)
		self.MonthText:SetSize(dateWidth1,self.TextHeight)
		self.MonthText:SetText(self.MonthText:GetText())
		self.Sep1:SetFont(self.Font)
		self.Sep1:SetLeft(dateWidth1+1)
		self.Sep1:SetSize(sepWidth,self.TextHeight)
		self.Sep1:SetText(self.Sep1:GetText())
		self.DayText:SetFont(self.Font)
		self.DayText:SetLeft(dateWidth1+sepWidth+1)
		self.DayText:SetSize(dateWidth1,self.TextHeight)
		self.DayText:SetText(self.DayText:GetText())
		self.Sep2:SetFont(self.Font)
		self.Sep2:SetLeft(dateWidth1*2+sepWidth+1)
		self.Sep2:SetSize(sepWidth,self.TextHeight)
		self.Sep2:SetText(self.Sep2:GetText())
		self.YearText:SetFont(self.Font)
		self.YearText:SetLeft(dateWidth1*2+sepWidth*2+1)
		self.YearText:SetSize(dateWidth2,self.TextHeight)
		self.YearText:SetText(self.YearText:GetText())
		self.Calendar:SetFont(self.Font)
		if self.TextHeight>=28 then
			self.CalendarIcon:SetSize(32,28)
			self.CalendarIcon:SetBackground(resourcePath.."/calendar32x28.jpg")
		elseif self.TextHeight>=20 then
			self.CalendarIcon:SetSize(23,20)
			self.CalendarIcon:SetBackground(resourcePath.."/calendar23x20.jpg")
		else
			self.CalendarIcon:SetSize(18,16)
			self.CalendarIcon:SetBackground(resourcePath.."/calendar18x16.jpg")
		end
		self.CalendarIcon:SetPosition(self.YearText:GetLeft()+self.YearText:GetWidth()+2,(self:GetHeight()-self.CalendarIcon:GetHeight())/2)
		if self.CalendarEnabled then
			self:SetWidth(self.YearText:GetLeft()+self.YearText:GetLeft()+2)
		else
			self:SetWidth(self.CalendarIcon:GetLeft()+self.CalendarIcon:GetWidth()+2)
		end
		self.Background:SetSize(self:GetWidth()-2,self.TextHeight)
	end
end
DateBox.SetEnabled=function(self,state)
	self.MonthText:SetEnabled(state)
	self.DayText:SetEnabled(state)
	self.YearText:SetEnabled(state)
end

DateBox.SetTrimColor=function(self,color)
	Turbine.UI.Control.SetBackColor(self,color)
end
DateBox.SetBackColor=function(self,color)
	self.Background:SetBackColor(color)
	self.MonthText:SetBackColor(color)
	self.Sep1:SetBackColor(color)
	self.DayText:SetBackColor(color)
	self.Sep2:SetBackColor(color)
	self.YearText:SetBackColor(color)
	self.Calendar:SetBackColor(color)
end

DateBox.SetForeColor=function(self,color)
	self.ForeColor=color;
	self.MonthText:SetForeColor(color)
	self.Sep1:SetForeColor(color)
	self.DayText:SetForeColor(color)
	self.Sep2:SetForeColor(color)
	self.YearText:SetForeColor(color)
	self.Calendar:SetForeColor(color)
end

DateBox.SizeChanged=function()
	-- this is a tough one since we allow resizing based on font
	-- perhaps we should auto adjust font to fit within the new size? if possible?
	-- or perhaps enforce minimum size based on font and allow control to grow with text controls centered?
end

DateBox.SetDateSeparator=function(self,dateSep)
	if dateSep==nil or dateSep=="" then dateSep="/" end
	if string.len(dateSep)>1 then dateSep=string.sub(dateSep,1) end
	self.Sep1:SetText(dateSep)
	self.Sep2:SetText(dateSep)
end

DateBox.GetValue=function()
	-- return a date formatted string "mm/dd/yyyy"
	local dataValue=string.sub("00"..self.MonthText:GetText(),-2)..self.DateSep..string.sub("00"..self.DayText:GetText(),-2)..self.DateSep..string.sub("0000"..self.YearText:GetText(),-4)
	return (dataValue);
end
DateBox.SetDay=function(self,day)
	day=tonumber(day)
	if day==nil then
		local date=Turbine.Engine:GetDate()
		day=date.Day
	end
	self.Day=day
	self.DayText:SetText(tostring(day))
	if self.DateChanged~=nil then self.DateChanged() end
end
DateBox.SetMonth=function(self,month)
	month=tonumber(month)
	if month==nil then
		local date=Turbine.Engine:GetDate()
		month=date.Month
	end
	self.Month=month
	self.MonthText:SetText(tostring(month))
	if self.DateChanged~=nil then self.DateChanged() end
end
DateBox.SetYear=function(self,year)
	year=tonumber(year)
	if year==nil then
		local date=Turbine.Engine:GetDate()
		year=date.Year
	end
	self.Year=year
	self.YearText:SetText(tostring(year))
	if self.DateChanged~=nil then self.DateChanged() end
end

DateBox.SetValue=function(self,value)
	if value==nil then
		local date=Turbine.Engine.GetDate()
		value=string.sub("00"..tostring(date.MonthText),-2).."/"..string.sub("00"..tostring(date.DayText),-2).."/"..string.sub("0000"..tostring(date.YearText),-4)
	end
	local month,day,year=string.match(value,"(%d+)%/(%d+)%/(%d+)")
	if month~=nil and day~=nil and year~=nil then
		self.MonthText:SetText(month)
		self.DayText:SetText(day)
		self.YearText:SetText(year)
	else
		self.MonthText:SetText("")
		self.DayText:SetText("")
		self.YearText:SetText("")
	end
	self.Month=month
	self.Day=day
	self.Year=year

	if self.DateChanged~=nil then self.DateChanged() end
end
